<?php
require_once __DIR__.'/initpro-kassa-php-sdk/autoload.php';

use Initpro\KassaSdk\Check;
use Initpro\KassaSdk\Client;
use Initpro\KassaSdk\Exception\SdkException;
use Initpro\KassaSdk\Payment;
use Initpro\KassaSdk\Position;
use Initpro\KassaSdk\GroupManager;
use Initpro\KassaSdk\TaxSystem;
use Initpro\KassaSdk\Vat;

class InitproKassa {
	private $registry;

	public function __construct($registry) {
		$this->registry = $registry;
	}

	public function __get($key) {
		return $this->registry->get($key);
	}

	public function __set($key, $value) {
		$this->registry->set($key, $value);
	}

	public function getTaxSystems() {
		return array(
			TaxSystem::COMMON,
			TaxSystem::SIMPLIFIED_IN,
			TaxSystem::SIMPLIFIED_IN_OUT,
			TaxSystem::UTOII,
			TaxSystem::UST,
			TaxSystem::PATENT
		);
	}

	public function getVatRates() {
		return array(
			Vat::RATE_NO,
			Vat::RATE_0,
			Vat::RATE_10,
			Vat::RATE_18,
			Vat::RATE_110,
			Vat::RATE_118,
		);
	}

	public function printCheck($orderID) {
		$this->load->model('checkout/order');
		$order = $this->model_checkout_order->getOrder($orderID);

		if (!in_array($order['payment_code'], $this->config->get('module_initpro_kassa_payment_codes'))) {
			return;
		}

		$statusID = $order['order_status_id'];

		if (in_array($statusID, $this->config->get('module_initpro_kassa_statuses_sell'))) {
			$intent = Check::INTENT_SELL;
			$reportType = 'sell';
		} else if (in_array($statusID, $this->config->get('module_initpro_kassa_statuses_return'))) {
			$intent = Check::INTENT_SELL_RETURN;
            $reportType = 'sell_return';
		} else {
			return;
		}

		$totals = $this->getOrderTotals($orderID);
		$additionalPrice = ($totals['tax'] + $totals['coupon'] + $totals['voucher']) / $totals['sub_total'];

		$taxSystem = intval($this->config->get('module_initpro_kassa_tax_system'));
		do {
            $taskID = uniqid(strval($orderID), true);
            $stmt = sprintf("SELECT * FROM " . DB_PREFIX . "initpro_kassa_report WHERE task_id = '%s'", $taskID);
            $query = $this->db->query($stmt);
        } while($query->num_rows !== 0);
		$check = new Check($taskID, $order['email'], $intent, $taxSystem);

		$total = 0;
		$stmt = sprintf("SELECT * FROM " . DB_PREFIX . "order_product WHERE order_id = %d", $orderID);
		$productVatRate = $this->config->get('module_initpro_kassa_vat_rate_product');
		foreach ($this->db->query($stmt)->rows as $product) {
			$productPrice = $product['price'] * (1 + $additionalPrice);
			$productPriceTotal = $productPrice * $product['quantity'];
			$check->addPosition(new Position(
				html_entity_decode($product['name']),
				$productPrice,
				floatval($product['quantity']),
				$productPriceTotal,
				abs(floatval($totals['coupon'])),
				new Vat($productVatRate)
			));
			$total += $productPriceTotal;
		}

		$shippingVatRate = $this->config->get('module_initpro_kassa_vat_rate_shipping');
		$check->addPosition(new Position(
			'Доставка',
			$totals['shipping'],
			1, // quantity
			$totals['shipping'],
			0, // discount
			new Vat($shippingVatRate)
		));

		$payment = new Payment(Payment::TYPE_CARD,$total + $totals['shipping']);
		$check->addPayment($payment);

		$client = new Client(
			$this->config->get('module_initpro_kassa_shop_id'),
			$this->config->get('module_initpro_kassa_secret_key')
		);
		$client->setHost($this->config->get('module_initpro_kassa_server_url'));
		$qm = new GroupManager($client);
		$qm->registerGroup('default', $this->config->get('module_initpro_kassa_queue_id'));
		$qm->setDefaultGroup('default');

        $lockResult = $this->db->query(sprintf('SELECT GET_LOCK(\'report_%s_%s\', 0) as l', $orderID, $intent))->row['l'];
        if(intval($lockResult) !== 1) {
            return;
        }
        $query = $this->db->query(sprintf('SELECT * FROM `%sinitpro_kassa_report` WHERE `order_id` = %d ORDER BY `id` DESC LIMIT 1',
            DB_PREFIX,
            $orderID));
        if($query->row['type'] === $reportType) {
            $this->db->query(sprintf("SELECT RELEASE_LOCK('report_%s_%s') as l", $orderID, $intent));
            return;
        }
		try {
			$data = $qm->putCheck($check);
            $this->db->query(sprintf(
                "INSERT INTO `%sinitpro_kassa_report` (`order_id`, `task_id`, `status`, `error`, `type`) VALUES ('%s', '%s', '%s', '%s', '%s')",
                DB_PREFIX,
                $orderID,
                $taskID,
                $data['state'],
                $data['state'] === 'error' ? $data['error_description'] : '',
                $reportType
            ));
		} catch (SdkException $e) {
			error_log(sprintf('Failed to print check: %s', $e->getMessage()));
		}
        $this->db->query(sprintf("SELECT RELEASE_LOCK('report_%s_%s') as l", $orderID, $intent));
	}

	private function getOrderTotals($orderID) {
		$stmt = "SELECT code, value FROM " . DB_PREFIX . "order_total WHERE order_id = %d ORDER BY sort_order";
		$rows = $this->db->query(sprintf($stmt, $orderID))->rows;
		$result = array(
			'coupon' => 0,
			'shipping' => 0,
			'sub_total' => 0,
			'tax' => 0,
			'voucher' => 0
		);
		foreach ($rows as $row) {
			if (array_key_exists($row['code'], $result)) {
				$result[$row['code']] += $row['value'];
			}
		}

		return $result;
	}
}
